import os

import numpy as np
import nibabel as nb
from brainio import brainio

from imlib.general.config import get_config_obj

transpositions = {
    "horizontal": (1, 0, 2),
    "coronal": (2, 0, 1),
    "sagittal": (2, 1, 0),
}


class AtlasError(Exception):
    pass


class Atlas(object):
    """
    A class to handle all the atlas data (including the
    """

    def __init__(self, config_path, dest_folder=""):
        config_obj = get_config_obj(config_path)
        self.atlas_conf = config_obj["atlas"]

        self.dest_folder = dest_folder

        self._pix_sizes = None  # cached to avoid reloading atlas
        self._data = None
        self._brain_data = None
        self._hemispheres_data = None

        self.original_orientation = self.atlas_conf["orientation"]
        if self.original_orientation != "horizontal":
            raise NotImplementedError(
                "Unknown orientation {}. Only horizontal supported so far".format(
                    self.original_orientation
                )
            )

    @property
    def pix_sizes(self):
        """
        Get the dictionary of x, y, z from the after loading it
        or if the atlas size is default, use the values from the config file

        :return: The dictionary of x, y, z pixel sizes
        """
        if self._pix_sizes is None:
            pixel_sizes = self.get_data().header.get_zooms()
            if pixel_sizes != (0, 0, 0):
                self._pix_sizes = {
                    axis: round(size * 1000, 3)  # convert to um
                    for axis, size in zip(("x", "y", "z"), pixel_sizes)
                }
            else:
                self._pix_sizes = self.get_pixel_sizes_from_config()
        return self._pix_sizes

    def get_path(self):
        """
        Get the path to the reference atlas

        :return: The atlas path
        :rtype: str
        """
        return self.get_atlas_element_path("atlas_name")

    def get_brain_path(self):
        return self.get_atlas_element_path("brain_name")

    def get_hemispheres_path(self):
        return self.get_atlas_element_path("hemispheres_name")

    def get_structures_path(self):
        return self.get_atlas_element_path("structures_name")

    def get_left_hemisphere_value(self):
        return int(self.atlas_conf["left_hemisphere_value"])

    def get_right_hemisphere_value(self):
        return int(self.atlas_conf["right_hemisphere_value"])

    def get_data(self):
        """
        Load the atlas and return it

        :return: The atlas (nifty image)
        """
        atlas_path = self.get_path()
        if self._data is None:
            self._data = brainio.load_nii(atlas_path)
        return self._data

    def load_all(self):
        if self._data is None:
            self._data = brainio.load_nii(self.get_path())
        if self._brain_data is None:
            self._brain_data = brainio.load_nii(self.get_brain_path())
        if self._hemispheres_data is None:
            self._hemispheres_data = brainio.load_nii(
                self.get_hemispheres_path()
            )

    def save_all(self):
        brainio.to_nii(self._data, self.get_dest_path("atlas_name"))
        brainio.to_nii(self._brain_data, self.get_dest_path("brain_name"))
        brainio.to_nii(
            self._hemispheres_data, self.get_dest_path("hemispheres_name")
        )

    # FIXME: should be just changing the header
    def _flip(self, nii_img, axis_idx):
        return nb.Nifti1Image(
            np.flip(nii_img.get_data(), axis_idx),
            nii_img.affine,
            nii_img.header,
        )

    def _flip_all(self, axis_idx):
        self._data = self._flip(self._data, axis_idx)
        self._brain_data = self._flip(self._brain_data, axis_idx)
        self._hemispheres_data = self._flip(self._hemispheres_data, axis_idx)

    def flip(self, axes):
        for axis_idx, flip_axis in enumerate(axes):
            if flip_axis:
                self._flip_all(axis_idx)

    def _transpose(self, nii_img, transposition):
        # FIXME: should be just changing the header
        data = np.transpose(nii_img.get_data(), transposition)
        data = np.swapaxes(data, 0, 1)
        return nb.Nifti1Image(data, nii_img.affine, nii_img.header)

    def _transpose_all(self, transposition):
        self._data = self._transpose(self._data, transposition)
        self._brain_data = self._transpose(self._brain_data, transposition)
        self._hemispheres_data = self._transpose(
            self._hemispheres_data, transposition
        )

    def reorientate_to_sample(self, sample_orientation):
        self._transpose_all(transpositions[sample_orientation])

    def get_dest_path(self, atlas_element_name):
        if not self.dest_folder:
            raise AtlasError(
                "Could not get destination path. "
                "Missing destination folder information"
            )
        return os.path.join(
            self.dest_folder, self.atlas_conf[atlas_element_name]
        )

    def make_atlas_scale_transformation_matrix(self):
        scale = self.pix_sizes
        transformation_matrix = np.eye(4)
        for i, axis in enumerate(("x", "y", "z")):
            transformation_matrix[i, i] = scale[axis] / 1000
        return transformation_matrix

    def get_pixel_sizes_from_config(self):
        """
        Get the dictionary of atlas pixel sizes from the config file.
        The dictionary is structured like this:
        {'x': x_pixel_size_in_um,
         'y': y_pixel_size_in_um,
         'z': z_pixel_size_in_um
        }

        :return: the pixel size dictionary
        :rtype: dict
        """
        return self.atlas_conf["pixel_size"]

    def get_atlas_element_path(self, config_entry_name):
        """
        Get the path to an 'element' of the atlas (i.e. the average brain,
        the atlas, or the hemispheres atlas)

        :param str config_entry_name: The name of the item to retrieve
        :return: The path to that atlas element on the filesystem
        :rtype: str
        """
        atlas_folder_name = os.path.expanduser(self.atlas_conf["base_folder"])
        atlas_element_filename = self.atlas_conf[config_entry_name]
        return os.path.abspath(
            os.path.normpath(
                os.path.join(atlas_folder_name, atlas_element_filename)
            )
        )



"""
registration_params
===================

Module to calculate volume of brain regions
"""

import numpy as np
import pandas as pd
import logging

from brainio import brainio
from imlib.pandas.misc import initialise_df
from imlib.general.config import get_config_obj


def load_structures_as_df(structures_file_path):
    return pd.read_csv(structures_file_path, sep=",", header=0, quotechar='"')


def get_voxel_volume(registration_config):
    config_obj = get_config_obj(registration_config)
    atlas_conf = config_obj["atlas"]
    atlas_pixel_sizes = atlas_conf["pixel_size"]
    x_pixel_size = float(atlas_pixel_sizes["x"])
    y_pixel_size = float(atlas_pixel_sizes["y"])
    z_pixel_size = float(atlas_pixel_sizes["z"])

    voxel_volume = x_pixel_size * y_pixel_size * z_pixel_size
    return voxel_volume


def lateralise_atlas(
    atlas, hemispheres, left_hemisphere_value=2, right_hemisphere_value=1
):
    atlas_left = atlas[hemispheres == left_hemisphere_value]
    atlas_right = atlas[hemispheres == right_hemisphere_value]
    return atlas_left, atlas_right


def get_lateralised_atlas(
    atlas_path,
    hemispheres_path,
    left_hemisphere_value=2,
    right_hemisphere_value=1,
):
    atlas = brainio.load_any(atlas_path)
    hemispheres = brainio.load_any(hemispheres_path)

    atlas_left, atlas_right = lateralise_atlas(
        atlas,
        hemispheres,
        left_hemisphere_value=left_hemisphere_value,
        right_hemisphere_value=right_hemisphere_value,
    )

    unique_vals_left, counts_left = np.unique(atlas_left, return_counts=True)
    unique_vals_right, counts_right = np.unique(
        atlas_right, return_counts=True
    )
    return unique_vals_left, unique_vals_right, counts_left, counts_right


class UnknownAtlasValue(Exception):
    pass


def atlas_value_to_name(atlas_value, structures_reference_df):
    line = structures_reference_df[
        structures_reference_df["id"] == atlas_value
    ]
    if len(line) == 0:
        raise UnknownAtlasValue(atlas_value)
    name = line["name"]
    return str(name.values[0])


def add_structure_volume_to_df(
    df,
    atlas_value,
    structures_reference_df,
    unique_vals_left,
    unique_vals_right,
    counts_left,
    counts_right,
    voxel_volume,
):

    name = atlas_value_to_name(atlas_value, structures_reference_df)

    try:
        left_index = np.where(unique_vals_left == atlas_value)[0][0]
        left_volume = counts_left[left_index] * voxel_volume
    except IndexError:
        logging.warning(
            "Atlas value: {} not found in registered atlas. "
            "Setting registered volume to 0.".format(atlas_value)
        )
        left_volume = 0

    try:
        right_index = np.where(unique_vals_right == atlas_value)[0][0]
        right_volume = counts_right[right_index] * voxel_volume
    except IndexError:
        logging.warning(
            "Atlas value: {} not found in registered atlas. "
            "Setting registered volume to 0.".format(atlas_value)
        )
        right_volume = 0

    df = df.append(
        {
            "structure_name": name,
            "left_volume_mm3": left_volume,
            "right_volume_mm3": right_volume,
            "total_volume_mm3": left_volume + right_volume,
        },
        ignore_index=True,
    )
    return df


def calculate_volumes(
    atlas_path,
    hemispheres_path,
    atlas_structures_path,
    registration_config,
    output_file,
    left_hemisphere_value=2,
    right_hemisphere_value=1,
):
    (
        unique_vals_left,
        unique_vals_right,
        counts_left,
        counts_right,
    ) = get_lateralised_atlas(
        atlas_path,
        hemispheres_path,
        left_hemisphere_value=left_hemisphere_value,
        right_hemisphere_value=right_hemisphere_value,
    )

    structures_reference_df = load_structures_as_df(atlas_structures_path)

    voxel_volume = get_voxel_volume(registration_config)
    voxel_volume_in_mm = voxel_volume / (1000 ** 3)
    df = initialise_df(
        "structure_name",
        "left_volume_mm3",
        "right_volume_mm3",
        "total_volume_mm3",
    )

    for atlas_value in unique_vals_left:
        if atlas_value is not 0:  # outside brain
            try:
                df = add_structure_volume_to_df(
                    df,
                    atlas_value,
                    structures_reference_df,
                    unique_vals_left,
                    unique_vals_right,
                    counts_left,
                    counts_right,
                    voxel_volume_in_mm,
                )

            except UnknownAtlasValue:
                print(
                    "Value: {} is not in the atlas structure reference file. "
                    "Not calculating the volume".format(atlas_value)
                )

    df.to_csv(output_file, index=False)



import os
import shutil
import tarfile
import urllib.request

from imlib.general.system import disk_free_gb
from imlib.general.config import get_config_obj
from amap.tools import source_files


class DownloadError(Exception):
    pass


def download_file(destination_path, file_url, filename):
    direct_download = True
    file_url = file_url.format(int(direct_download))
    print(f"Downloading file: {filename}")
    with urllib.request.urlopen(file_url) as response:
        with open(destination_path, "wb") as outfile:
            shutil.copyfileobj(response, outfile)


def extract_file(tar_file_path, destination_path):
    tar = tarfile.open(tar_file_path)
    tar.extractall(path=destination_path)
    tar.close()


# TODO: check that intermediate folders exist
def download(
    download_path,
    url,
    file_name,
    install_path=None,
    download_requires=None,
    extract_requires=None,
):
    if not os.path.exists(os.path.dirname(download_path)):
        raise DownloadError(
            f"Could not find directory '{os.path.dirname(download_path)}' "
            f"to download file: {file_name}"
        )

    if (download_requires is not None) and (
        disk_free_gb(os.path.dirname(download_path)) < download_requires
    ):
        raise DownloadError(
            f"Insufficient disk space in {os.path.dirname(download_path)} to"
            f"download file: {file_name}"
        )

    if install_path is not None:
        if not os.path.exists(install_path):
            raise DownloadError(
                f"Could not find directory '{install_path}' "
                f"to extract file: {file_name}"
            )

        if (extract_requires is not None) and (
            disk_free_gb(install_path) < extract_requires
        ):
            raise DownloadError(
                f"Insufficient disk space in {install_path} to"
                f"extract file: {file_name}"
            )
            raise DownloadError(
                "Insufficient disk space in {} to install atlas".format(
                    install_path
                )
            )

    download_file(download_path, url, file_name)
    if install_path is not None:
        extract_file(download_path, install_path)
        os.remove(download_path)


def amend_cfg(new_atlas_folder=None, atlas=None):
    """
    Updates the registration config file to point to the correct atlas path
    :param new_atlas_folder:
    """
    print("Ensuring custom config file is correct")

    original_config = source_files.source_config()
    new_config = source_files.source_custom_config()
    if new_atlas_folder is not None:
        write_atlas_to_cfg(
            new_atlas_folder, atlas, original_config, new_config
        )


def write_atlas_to_cfg(atlas_folder, atlas, orig_config, custom_config):
    config_obj = get_config_obj(orig_config)
    atlas_conf = config_obj["atlas"]
    orig_base_directory = atlas_conf["base_folder"]

    with open(orig_config, "r") as in_conf:
        data = in_conf.readlines()
    for i, line in enumerate(data):
        data[i] = line.replace(
            f"base_folder = '{orig_base_directory}",
            f"base_folder = '{os.path.join(atlas_folder, atlas)}",
        )
    with open(custom_config, "w") as out_conf:
        out_conf.writelines(data)



"""
brain_registration
==================

The module to actually start the registration
"""


import logging


from imlib.general.system import (
    safe_execute_command,
    SafeExecuteCommandError,
)

from imlib.general.exceptions import RegistrationError, SegmentationError


class BrainRegistration(object):
    """
    A class to register brains using the nifty_reg set of binaries
    """

    def __init__(
        self,
        registration_config,
        paths,
        registration_params,
        n_processes=None,
    ):
        self.registration_config = registration_config
        self.paths = paths
        self.reg_params = registration_params
        if n_processes is not None:
            self.n_processes = n_processes
            self._prepare_openmp_thread_flag()
        else:
            self.n_processes = None

        self.dataset_img_path = paths.tmp__downsampled_filtered
        self.brain_of_atlas_img_path = paths.brain_filtered
        self.atlas_img_path = paths.annotations
        self.hemispheres_img_path = paths.hemispheres

    def _prepare_openmp_thread_flag(self):
        self.openmp_flag = "-omp {}".format(self.n_processes)

    def _prepare_affine_reg_cmd(self):
        cmd = "{} {} -flo {} -ref {} -aff {} -res {}".format(
            self.reg_params.affine_reg_program_path,
            self.reg_params.format_affine_params().strip(),
            self.brain_of_atlas_img_path,
            self.dataset_img_path,
            self.paths.affine_matrix_path,
            self.paths.tmp__affine_registered_atlas_brain_path,
        )

        if self.n_processes is not None:
            cmd += " " + self.openmp_flag
        return cmd

    def register_affine(self):
        """
        Performs affine registration of the average brain of the atlas to the
        sample using nifty_reg reg_aladin

        :return:
        :raises RegistrationError: If any error was detected during
            registration.
        """
        try:
            safe_execute_command(
                self._prepare_affine_reg_cmd(),
                self.paths.tmp__affine_log_file_path,
                self.paths.tmp__affine_error_path,
            )
        except SafeExecuteCommandError as err:
            raise RegistrationError(
                "Affine registration failed; {}".format(err)
            )

    def _prepare_freeform_reg_cmd(self):
        cmd = "{} {} -aff {} -flo {} -ref {} -cpp {} -res {}".format(
            self.reg_params.freeform_reg_program_path,
            self.reg_params.format_freeform_params().strip(),
            self.paths.affine_matrix_path,
            self.brain_of_atlas_img_path,
            self.dataset_img_path,
            self.paths.control_point_file_path,
            self.paths.tmp__freeform_registered_atlas_brain_path,
        )

        if self.n_processes is not None:
            cmd += " " + self.openmp_flag
        return cmd

    def register_freeform(self):
        """
        Performs freeform (elastic) registration of the average brain of the
        atlas to the sample brain using nifty_reg reg_f3d

        :return:
        :raises RegistrationError: If any error was detected during
            registration.
        """
        try:
            safe_execute_command(
                self._prepare_freeform_reg_cmd(),
                self.paths.tmp__freeform_log_file_path,
                self.paths.tmp__freeform_error_file_path,
            )
        except SafeExecuteCommandError as err:
            raise RegistrationError(
                "Freeform registration failed; {}".format(err)
            )

    def generate_inverse_transforms(self):
        self.generate_inverse_affine()
        self.register_inverse_freeform()

    def _prepare_invert_affine_cmd(self):
        cmd = "{} -invAff {} {}".format(
            self.reg_params.transform_program_path,
            self.paths.affine_matrix_path,
            self.paths.invert_affine_matrix_path,
        )
        return cmd

    def generate_inverse_affine(self):
        """
        Inverts the affine transform to allow for quick registration of the
        sample onto the atlas
        :return:
        :raises RegistrationError: If any error was detected during
            registration.

        """
        logging.debug("Generating inverse affine transform")
        try:
            safe_execute_command(
                self._prepare_invert_affine_cmd(),
                self.paths.tmp__invert_affine_log_file,
                self.paths.tmp__invert_affine_error_file,
            )
        except SafeExecuteCommandError as err:
            raise RegistrationError(
                "Generation of inverted affine transform failed; "
                "{}".format(err)
            )

    def _prepare_inverse_freeform_reg_cmd(self):
        cmd = "{} {} -aff {} -flo {} -ref {} -cpp {} -res {}".format(
            self.reg_params.freeform_reg_program_path,
            self.reg_params.format_freeform_params().strip(),
            self.paths.invert_affine_matrix_path,
            self.dataset_img_path,
            self.brain_of_atlas_img_path,
            self.paths.inverse_control_point_file_path,
            self.paths.tmp__inverse_freeform_registered_atlas_brain_path,
        )

        if self.n_processes is not None:
            cmd += " " + self.openmp_flag
        return cmd

    def register_inverse_freeform(self):
        """
        Performs freeform (elastic) registration of the sample to the
        atlas using nifty_reg reg_f3d

        :return:
        :raises RegistrationError: If any error was detected during
            registration.
        """
        logging.debug("Registering sample to atlas")

        try:
            safe_execute_command(
                self._prepare_inverse_freeform_reg_cmd(),
                self.paths.tmp__inverse_freeform_log_file_path,
                self.paths.tmp__inverse_freeform_error_file_path,
            )
        except SafeExecuteCommandError as err:
            raise RegistrationError(
                "Inverse freeform registration failed; {}".format(err)
            )

    def _prepare_segmentation_cmd(self, floating_image_path, dest_img_path):
        cmd = "{} {} -cpp {} -flo {} -ref {} -res {}".format(
            self.reg_params.segmentation_program_path,
            self.reg_params.format_segmentation_params().strip(),
            self.paths.control_point_file_path,
            floating_image_path,
            self.dataset_img_path,
            dest_img_path,
        )
        return cmd

    def segment(self):
        """
        Registers the atlas to the sample (propagates the transformation
        computed for the average brain of the atlas to the atlas itself).


        :return:
        :raises SegmentationError: If any error was detected during the
            propagation.
        """
        try:
            safe_execute_command(
                self._prepare_segmentation_cmd(
                    self.atlas_img_path, self.paths.registered_atlas_img_path
                ),
                self.paths.tmp__segmentation_log_file,
                self.paths.tmp__segmentation_error_file,
            )
        except SafeExecuteCommandError as err:
            SegmentationError("Segmentation failed; {}".format(err))

    def register_hemispheres(self):
        """
        Registers the hemispheres atlas to the sample (propagates the
        transformation computed for the average brain of the atlas to the
        hemispheres atlas itself).

        :return:
        :raises RegistrationError: If any error was detected during the
            propagation.
        """
        try:
            safe_execute_command(
                self._prepare_segmentation_cmd(
                    self.hemispheres_img_path,
                    self.paths.registered_hemispheres_img_path,
                ),
                self.paths.tmp__segmentation_log_file,
                self.paths.tmp__segmentation_error_file,
            )
        except SafeExecuteCommandError as err:
            SegmentationError("Segmentation failed; {}".format(err))



import napari
import numpy as np
from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
from natsort import natsorted
from napari.utils.io import magic_imread
from pathlib import Path
from vispy.color import Colormap
from brainio import brainio
from imlib.general.system import get_sorted_file_paths, get_text_lines
from imlib.general.config import get_config_obj
from imlib.IO.structures import load_structures_as_df
from imlib.anatomy.structures.structures_tree import (
    atlas_value_to_name,
    UnknownAtlasValue,
)
from amap.utils.paths import Paths
from amap.tools.source_files import get_structures_path

label_red = Colormap([[0.0, 0.0, 0.0, 0.0], [1.0, 1.0, 1.0, 1.0]])


def parser():
    parser = ArgumentParser(formatter_class=ArgumentDefaultsHelpFormatter)
    parser = cli_parse(parser)
    return parser


def cli_parse(parser):
    cli_parser = parser.add_argument_group("Visualisation options")

    cli_parser.add_argument(
        dest="amap_directory",
        type=str,
        help="Path to the amap output directory..",
    )

    cli_parser.add_argument(
        "-r" "--raw",
        dest="raw",
        action="store_true",
        help="Display the raw image (as a virtual stack) "
        "rather than the downsampled",
    )
    cli_parser.add_argument(
        "-c",
        "--raw-channels",
        dest="raw_channels",
        type=str,
        nargs="+",
        help="Paths to N additional raw channels to view. Will only work if "
        "using the raw image viewer.",
    )
    cli_parser.add_argument(
        "-m" "--memory",
        dest="memory",
        action="store_true",
        help="Load data into RAM. ",
    )

    return parser


def get_most_recent_log(directory, log_pattern="amap*.log"):
    """
    Returns the most recent amap log file (for parsing of arguments)
    :param directory:
    :param log_pattern: String pattern that defines the log
    :return: Path to the most recent log file
    """
    directory = Path(directory)
    return natsorted(directory.glob(log_pattern))[-1]


def read_log_file(
    log_file,
    log_entries_to_get=[
        "x_pixel_um",
        "y_pixel_um",
        "z_pixel_um",
        "image_paths",
        "registration_config",
    ],
    separator=": ",
):
    """
    Reads an amap log file, and returns a dict of entries corresponding to
    "log_entries_to_get"
    :param log_file: Path to the log file
    :param log_entries_to_get: List of strings corresponding to entries
    in the log file
    :param separator: Separator between the log item label and the entry.
    Default: ": "
    :return: A dict of the entries and labels
    """
    lines = get_text_lines(log_file)
    entries = {}
    for line in lines:
        for entry in log_entries_to_get:
            if line.startswith(entry):
                entries[entry] = line.strip(entry + separator)

    return entries


def prepare_load_nii(nii_path, memory=False):
    """
    Transforms a nii file into the same coordinate space as the raw data
    :param nii_path: Path to the nii file
    :param memory: Load data into memory
    :return: Numpy array in the correct coordinate space
    """
    nii_path = str(nii_path)
    image = brainio.load_any(nii_path, as_numpy=memory)
    image = np.swapaxes(image, 2, 0)
    return image


def load_additional_downsampled_images(
    viewer,
    amap_directory,
    paths,
    search_string="downsampled_",
    extension=".nii",
    memory=False,
):
    """
    Loads additional downsampled (i.e. from nii) images into napari viewer
    :param viewer: Napari viewer object
    :param amap_directory: Directory containing images
    :param paths: amap paths object
    :param search_string: String that defines the images.
    Default: "downampled_"
    :param extension: File extension of the downsampled images. Default: ".nii"
    :param memory: Load data into memory
    """

    amap_directory = Path(amap_directory)

    for file in amap_directory.iterdir():
        if (
            (file.suffix == ".nii")
            and file.name.startswith(search_string)
            and file != Path(paths.downsampled_brain_path)
            and file != Path(paths.tmp__downsampled_filtered)
        ):
            print(
                f"Found additional downsampled image: {file.name}, "
                f"adding to viewer"
            )
            name = file.name.strip(search_string).strip(extension)
            viewer.add_image(
                prepare_load_nii(file, memory=memory), name=name,
            )


def add_raw_image(viewer, image_path, name):
    """
    Add a raw image (as a virtual stack) to the napari viewer
    :param viewer: Napari viewer object
    :param image_path: Path to the raw data
    :param str name: Name to give the data
    """
    paths = get_sorted_file_paths(image_path, file_extension=".tif")
    images = magic_imread(paths, use_dask=True, stack=True)
    viewer.add_image(images, name=name, opacity=0.6, blending="additive")


def get_image_scales(log_entries, config_file):
    """
    Returns the scaling from downsampled data to raw data
    :param log_entries: Entries parsed from the log file
    :param config_file: Path to the amap config file
    :return: Tuple of scaling factors
    """
    config_obj = get_config_obj(config_file)
    atlas_conf = config_obj["atlas"]
    pixel_sizes = atlas_conf["pixel_size"]
    x_scale = float(pixel_sizes["x"]) / float(log_entries["x_pixel_um"])
    y_scale = float(pixel_sizes["y"]) / float(log_entries["y_pixel_um"])
    z_scale = float(pixel_sizes["z"]) / float(log_entries["z_pixel_um"])
    return z_scale, y_scale, x_scale


def display_raw(viewer, args):
    """
    Display raw data
    :param viewer:
    :param args:
    :return:
    """
    print(
        "Starting raw image viewer. Streaming full-resolution data."
        " This may be slow."
    )
    log_entries = read_log_file(get_most_recent_log(args.amap_directory))
    config_file = Path(args.amap_directory, "config.conf")
    image_scales = get_image_scales(log_entries, config_file)
    add_raw_image(viewer, log_entries["image_paths"], name="Raw data")

    if args.raw_channels:
        for raw_image in args.raw_channels:
            name = Path(raw_image).name
            print(f"Found additional raw image to add to viewer: " f"{name}")
            add_raw_image(viewer, raw_image, name=name)

    return image_scales


def display_downsampled(viewer, args, paths):
    """
    Display downsampled data
    :param viewer:
    :param args:
    :param paths:
    :return:
    """
    image_scales = (1, 1, 1)
    load_additional_downsampled_images(
        viewer, args.amap_directory, paths, memory=args.memory
    )

    viewer.add_image(
        prepare_load_nii(paths.downsampled_brain_path, memory=args.memory),
        name="Downsampled raw data",
    )

    return image_scales


def display_registration(
    viewer, atlas, boundaries, image_scales, memory=False
):
    """
    Display results of the registration
    :param viewer: napari viewer object
    :param atlas: Annotations in sample space
    :param boundaries: Annotation boundaries in sample space
    :param tuple image_scales: Scaling of images from annotations -> data
    :param memory: Load data into memory
    """
    viewer.add_image(
        prepare_load_nii(boundaries, memory=memory),
        name="Outlines",
        contrast_limits=[0, 1],
        colormap=("label_red", label_red),
        scale=image_scales,
    )

    # labels added last so on top
    labels = viewer.add_labels(
        prepare_load_nii(atlas, memory=memory),
        name="Annotations",
        opacity=0.2,
        scale=image_scales,
    )
    return labels


def main():
    print("Starting amap viewer")
    args = parser().parse_args()

    structures_path = get_structures_path()
    structures_df = load_structures_as_df(structures_path)

    if not args.memory:
        print(
            "By default amap_vis does not load data into memory. "
            "To speed up visualisation, use the '-m' flag. Be aware "
            "this will make the viewer slower to open initially."
        )

    paths = Paths(args.amap_directory)
    with napari.gui_qt():
        v = napari.Viewer(title="amap viewer")
        if (
            Path(paths.registered_atlas_path).exists()
            and Path(paths.boundaries_file_path).exists()
        ):

            if args.raw:
                image_scales = display_raw(v, args)

            else:
                if Path(paths.downsampled_brain_path).exists():
                    image_scales = display_downsampled(v, args, paths)

                else:
                    raise FileNotFoundError(
                        f"The downsampled image: "
                        f"{paths.downsampled_brain_path} could not be found. "
                        f"Please ensure this is the correct "
                        f"directory and that amap has completed. "
                    )

            labels = display_registration(
                v,
                paths.registered_atlas_path,
                paths.boundaries_file_path,
                image_scales,
                memory=args.memory,
            )

            @labels.mouse_move_callbacks.append
            def get_connected_component_shape(layer, event):
                val = layer.get_value()
                if val != 0 and val is not None:
                    try:
                        region = atlas_value_to_name(val, structures_df)
                        msg = f"{region}"
                    except UnknownAtlasValue:
                        msg = "Unknown region"
                else:
                    msg = "No label here!"
                layer.help = msg

        else:
            raise FileNotFoundError(
                f"The directory: '{args.amap_directory}' does not "
                f"appear to be complete. Please ensure this is the correct "
                f"directory and that amap has completed."
            )


if __name__ == "__main__":
    main()



"""
source_files
===============

Return location of specific files specific to amap

"""


from pkg_resources import resource_filename
from amap.config.atlas import Atlas


def source_config():
    return resource_filename("amap", "amap.conf")


def source_custom_config():
    return resource_filename("amap", "amap.conf.custom")


def get_structures_path(config=None):
    if config is None:
        config = source_custom_config()
    atlas = Atlas(config)
    return atlas.get_structures_path()



import os
import logging
import numpy as np

from amap.register.brain_processor import BrainProcessor


def save_downsampled_image(
    image,
    name,
    output_folder,
    atlas,
    x_pixel_um=0.02,
    y_pixel_um=0.02,
    z_pixel_um=0.05,
    orientation="coronal",
    n_free_cpus=2,
    sort_input_file=False,
    load_parallel=True,
):
    """

    :param image: Image to be downsampled.
    :param str name: Name of the image (for the filename and logging)
    :param output_folder: Where the image is to be saved
    :param atlas: atlas config
    :param x_pixel_um: pixel spacing in the first dimension
    :param y_pixel_um: pixel spacing in the second dimension
    :param z_pixel_um: pixel spacing in the second dimension
    :param orientation: anatomical orientation of the image
    :param n_free_cpus: how many cpus to leave free when loading data
    :param sort_input_file: Should the data be naturally sorted
    (e.g. file paths)
    :param load_parallel: If possible, load the data in parallel
    """

    logging.info(f"Downsampling image: {name}")
    logging.info("Loading data")
    brain = BrainProcessor(
        atlas,
        image,
        output_folder,
        x_pixel_um,
        y_pixel_um,
        z_pixel_um,
        original_orientation=orientation,
        load_parallel=load_parallel,
        sort_input_file=sort_input_file,
        load_atlas=False,
        n_free_cpus=n_free_cpus,
    )

    downsampled_brain_path = os.path.join(
        output_folder, f"downsampled_{name}.nii"
    )

    brain.target_brain = brain.target_brain.astype(np.uint16, copy=False)
    logging.info("Saving downsampled image")
    brain.save(downsampled_brain_path)
    del brain



import os
import logging
import tempfile
from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
from datetime import datetime
from fancylog import fancylog
from pathlib import Path
from shutil import copyfile

from micrometa.micrometa import SUPPORTED_METADATA_TYPES
from imlib.general.system import ensure_directory_exists
from imlib.general.numerical import check_positive_int, check_positive_float
from imlib.image.metadata import define_pixel_sizes
from imlib.general.config import get_config_obj

from amap.download.cli import atlas_parser, download_directory_parser
from amap.main import main as register
from amap.tools import source_files
from amap.download import atlas as atlas_download
from amap.download.download import amend_cfg
import amap as program_for_log


temp_dir = tempfile.TemporaryDirectory()
temp_dir_path = temp_dir.name


def register_cli_parser():
    parser = ArgumentParser(formatter_class=ArgumentDefaultsHelpFormatter)
    parser = cli_parse(parser)
    parser = visualisation_parser(parser)
    parser = registration_parse(parser)
    parser = pixel_parser(parser)
    parser = geometry_parser(parser)
    parser = misc_parse(parser)
    parser = atlas_parser(parser)
    parser = download_directory_parser(parser)

    return parser


def cli_parse(parser):
    cli_parser = parser.add_argument_group("amap registration options")

    cli_parser.add_argument(
        dest="image_paths",
        type=str,
        help="Path to the directory of the image files. Can also be a text"
        "file pointing to the files.",
    )

    cli_parser.add_argument(
        dest="registration_output_folder",
        type=str,
        help="Directory to save the cubes into",
    )

    cli_parser.add_argument(
        "-d",
        "--downsample",
        dest="downsample_images",
        type=str,
        nargs="+",
        help="Paths to N additional channels to downsample to the same "
        "coordinate space. ",
    )
    return parser


def misc_parse(parser):
    misc_parser = parser.add_argument_group("Misc options")
    misc_parser.add_argument(
        "--n-free-cpus",
        dest="n_free_cpus",
        type=check_positive_int,
        default=4,
        help="The number of CPU cores on the machine to leave "
        "unused by the program to spare resources.",
    )

    misc_parser.add_argument(
        "--debug",
        dest="debug",
        action="store_true",
        help="Debug mode. Will increase verbosity of logging and save all "
        "intermediate files for diagnosis of software issues.",
    )
    misc_parser.add_argument(
        "--metadata",
        dest="metadata",
        type=Path,
        help="Path to the metadata file. Supported formats are '{}'.".format(
            SUPPORTED_METADATA_TYPES
        ),
    )
    return parser


def pixel_parser(parser):
    pixel_opt_parser = parser.add_argument_group(
        "Options to define pixel sizes of raw data"
    )
    pixel_opt_parser.add_argument(
        "-x",
        "--x-pixel-um",
        dest="x_pixel_um",
        type=check_positive_float,
        help="Pixel spacing of the data in the first "
        "dimension, specified in um.",
    )
    pixel_opt_parser.add_argument(
        "-y",
        "--y-pixel-um",
        dest="y_pixel_um",
        type=check_positive_float,
        help="Pixel spacing of the data in the second "
        "dimension, specified in um.",
    )
    pixel_opt_parser.add_argument(
        "-z",
        "--z-pixel-um",
        dest="z_pixel_um",
        type=check_positive_float,
        help="Pixel spacing of the data in the third "
        "dimension, specified in um.",
    )
    return parser


def geometry_parser(parser):
    geometry_opt_parser = parser.add_argument_group(
        "Options to define size/shape/orientation of data"
    )
    geometry_opt_parser.add_argument(
        "--orientation",
        type=str,
        choices=("coronal", "sagittal", "horizontal"),
        default="coronal",
        help="The orientation of the sample brain. "
        "This is used to transpose the atlas"
        "into the same orientation as the brain.",
    )

    # Warning: atlas reference
    geometry_opt_parser.add_argument(
        "--flip-x",
        dest="flip_x",
        action="store_true",
        help="If the supplied data does not match the NifTI standard "
        "orientation (origin is the most ventral, posterior, left voxel),"
        "then the atlas will be flipped to match the input data",
    )
    geometry_opt_parser.add_argument(
        "--flip-y",
        dest="flip_y",
        action="store_true",
        help="If the supplied data does not match the NifTI standard "
        "orientation (origin is the most ventral, posterior, left voxel),"
        "then the atlas will be flipped to match the input data",
    )
    geometry_opt_parser.add_argument(
        "--flip-z",
        dest="flip_z",
        action="store_true",
        help="If the supplied data does not match the NifTI standard "
        "orientation (origin is the most ventral, posterior, left voxel),"
        "then the atlas will be flipped to match the input data",
    )

    return parser


def visualisation_parser(parser):
    vis_parser = parser.add_argument_group(
        "Options relating to registration visualisation"
    )

    vis_parser.add_argument(
        "--no-boundaries",
        dest="no_boundaries",
        action="store_true",
        help="Do not precompute the outline images (if you don't want to"
        " use amap_vis",
    )
    return parser


def registration_parse(parser):
    registration_opt_parser = parser.add_argument_group("Registration options")
    registration_opt_parser.add_argument(
        "--registration-config",
        dest="registration_config",
        type=str,
        help="To supply your own, custom registration configuration file.",
    )
    registration_opt_parser.add_argument(
        "--sort-input-file",
        dest="sort_input_file",
        action="store_true",
        help="If set to true, the input text file will be sorted using "
        "natural sorting. This means that the file paths will be "
        "sorted as would be expected by a human and "
        "not purely alphabetically",
    )

    registration_opt_parser.add_argument(
        "--no-save-downsampled",
        dest="no_save_downsampled",
        action="store_true",
        help="Dont save the downsampled brain before filtering.",
    )

    registration_opt_parser.add_argument(
        "--affine-n-steps",
        dest="affine_n_steps",
        type=check_positive_int,
        default=6,
    )
    registration_opt_parser.add_argument(
        "--affine-use-n-steps",
        dest="affine_use_n_steps",
        type=check_positive_int,
        default=5,
    )
    registration_opt_parser.add_argument(
        "--freeform-n-steps",
        dest="freeform_n_steps",
        type=check_positive_int,
        default=6,
    )
    registration_opt_parser.add_argument(
        "--freeform-use-n-steps",
        dest="freeform_use_n_steps",
        type=check_positive_int,
        default=4,
    )
    registration_opt_parser.add_argument(
        "--bending-energy-weight",
        dest="bending_energy_weight",
        type=check_positive_float,
        default=0.95,
    )
    registration_opt_parser.add_argument(
        "--grid-spacing-x", dest="grid_spacing_x", type=int, default=-10
    )
    registration_opt_parser.add_argument(
        "--smoothing-sigma-reference",
        dest="smoothing_sigma_reference",
        type=float,
        default=-1.0,
    )
    registration_opt_parser.add_argument(
        "--smoothing-sigma-floating",
        dest="smoothing_sigma_floating",
        type=float,
        default=-1.0,
    )
    registration_opt_parser.add_argument(
        "--histogram-n-bins-floating",
        dest="histogram_n_bins_floating",
        type=check_positive_int,
        default=128,
    )
    registration_opt_parser.add_argument(
        "--histogram-n-bins-reference",
        dest="histogram_n_bins_reference",
        type=check_positive_int,
        default=128,
    )
    return parser


def check_atlas_install():
    """
    Checks whether the atlas directory exists, and whether it's empty or not.
    :return: Whether the directory exists, and whether the files also exist
    """
    dir_exists = False
    files_exist = False
    cfg_file_path = source_files.source_custom_config()
    if os.path.exists(cfg_file_path):
        config_obj = get_config_obj(cfg_file_path)
        atlas_conf = config_obj["atlas"]
        atlas_directory = atlas_conf["base_folder"]
        if os.path.exists(atlas_directory):
            dir_exists = True
            if not os.listdir(atlas_directory) == []:
                files_exist = True

    return dir_exists, files_exist


def copy_registration_config(registration_config, output_directory):
    destination = Path(output_directory, "config.conf")
    copyfile(registration_config, destination)


def prep_registration(args):
    logging.info("Checking whether the atlas exists")
    _, atlas_files_exist = check_atlas_install()
    if not atlas_files_exist:
        logging.warning("Atlas does not exist, downloading.")
        if args.download_path is None:
            args.download_path = os.path.join(temp_dir_path, "atlas.tar.gz")
        atlas_download.main(args.atlas, args.install_path, args.download_path)
        amend_cfg(
            new_atlas_folder=args.install_path, atlas=args.atlas,
        )
    if args.registration_config is None:
        args.registration_config = source_files.source_custom_config()

    logging.debug("Making registration directory")
    ensure_directory_exists(args.registration_output_folder)

    logging.debug("Copying registration config to output directory")
    copy_registration_config(
        args.registration_config, args.registration_output_folder
    )

    additional_images_downsample = {}
    if args.downsample_images:
        for idx, images in enumerate(args.downsample_images):
            name = Path(images).name
            additional_images_downsample[name] = images

    return args, additional_images_downsample


def make_paths_absolute(args):
    args.image_paths = os.path.abspath(args.image_paths)
    args.registration_output_folder = os.path.abspath(
        args.registration_output_folder
    )
    args.registration_config = os.path.abspath(args.registration_config)
    return args


def run():
    start_time = datetime.now()
    args = register_cli_parser().parse_args()
    args = define_pixel_sizes(args)

    args, additional_images_downsample = prep_registration(args)
    args = make_paths_absolute(args)

    fancylog.start_logging(
        args.registration_output_folder,
        program_for_log,
        variables=[args],
        verbose=args.debug,
        log_header="AMAP LOG",
        multiprocessing_aware=False,
    )

    logging.info("Starting registration")
    register(
        args.registration_config,
        args.image_paths,
        args.registration_output_folder,
        x_pixel_um=args.x_pixel_um,
        y_pixel_um=args.y_pixel_um,
        z_pixel_um=args.z_pixel_um,
        orientation=args.orientation,
        flip_x=args.flip_x,
        flip_y=args.flip_y,
        flip_z=args.flip_z,
        affine_n_steps=args.affine_n_steps,
        affine_use_n_steps=args.affine_use_n_steps,
        freeform_n_steps=args.freeform_n_steps,
        freeform_use_n_steps=args.freeform_use_n_steps,
        bending_energy_weight=args.bending_energy_weight,
        grid_spacing_x=args.grid_spacing_x,
        smoothing_sigma_reference=args.smoothing_sigma_reference,
        smoothing_sigma_floating=args.smoothing_sigma_floating,
        histogram_n_bins_floating=args.histogram_n_bins_floating,
        histogram_n_bins_reference=args.histogram_n_bins_reference,
        sort_input_file=args.sort_input_file,
        n_free_cpus=args.n_free_cpus,
        save_downsampled=not (args.no_save_downsampled),
        boundaries=not (args.no_boundaries),
        additional_images_downsample=additional_images_downsample,
        debug=args.debug,
    )

    logging.info("Finished. Total time taken: %s", datetime.now() - start_time)


def gui():
    from gooey import Gooey

    @Gooey(
        program_name="amap",
        default_size=(1000, 1000),
        required_cols=1,
        optional_cols=3,
    )
    def run_gui():
        run()

    run_gui()


def main():
    run()


if __name__ == "__main__":
    run()



import os
import tempfile
from pathlib import Path

from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter

from amap.download import atlas
from amap.download.download import amend_cfg

home = str(Path.home())
DEFAULT_DOWNLOAD_DIRECTORY = os.path.join(home, ".amap", "atlas")
temp_dir = tempfile.TemporaryDirectory()
temp_dir_path = temp_dir.name


def download_directory_parser(parser):
    parser.add_argument(
        "--install-path",
        dest="install_path",
        type=str,
        default=DEFAULT_DOWNLOAD_DIRECTORY,
        help="The path to install files to.",
    )
    parser.add_argument(
        "--download-path",
        dest="download_path",
        type=str,
        help="The path to download files into.",
    )
    parser.add_argument(
        "--no-amend-config",
        dest="no_amend_config",
        action="store_true",
        help="Don't amend the config file",
    )
    return parser


def atlas_parser(parser):
    parser.add_argument(
        "--no-atlas",
        dest="no_atlas",
        action="store_true",
        help="Don't download the atlas",
    )
    parser.add_argument(
        "--atlas",
        dest="atlas",
        type=str,
        default="allen_2017_10um",
        help="The atlas to use",
    )
    return parser


def download_parser():
    parser = ArgumentParser(formatter_class=ArgumentDefaultsHelpFormatter)
    parser = atlas_parser(parser)
    parser = download_directory_parser(parser)
    return parser


def main():
    args = download_parser().parse_args()
    if args.download_path is None:
        args.download_path = os.path.join(temp_dir_path, "atlas.tar.gz")
    if not args.no_atlas:
        atlas.main(args.atlas, args.install_path, args.download_path)
    if not args.no_amend_config:
        amend_cfg(
            new_atlas_folder=args.install_path, atlas=args.atlas,
        )


if __name__ == "__main__":
    main()



import os


class Paths:
    """
    A single class to hold all file paths that amap may need. Any paths
    prefixed with "tmp__" refer to internal intermediate steps, and will be
    deleted if "--debug" is not used.
    """

    def __init__(self, registration_output_folder):
        self.registration_output_folder = registration_output_folder
        self.make_reg_paths()

    def make_reg_paths(self):
        self.brain_filtered = self.make_reg_path("brain_filtered.nii")

        self.hemispheres = self.make_reg_path("hemispheres.nii")

        self.annotations = self.make_reg_path("annotations.nii")

        self.downsampled_brain_path = self.make_reg_path("downsampled.nii")
        self.tmp__downsampled_filtered = self.make_reg_path(
            "downsampled_filtered.nii"
        )
        self.registered_atlas_path = self.make_reg_path("registered_atlas.nii")
        self.hemispheres_atlas_path = self.make_reg_path(
            "registered_hemispheres.nii"
        )
        self.volume_csv_path = self.make_reg_path("volumes.csv")

        self.tmp__affine_registered_atlas_brain_path = self.make_reg_path(
            "affine_registered_atlas_brain.nii"
        )
        self.tmp__freeform_registered_atlas_brain_path = self.make_reg_path(
            "freeform_registered_atlas_brain.nii"
        )
        self.tmp__inverse_freeform_registered_atlas_brain_path = self.make_reg_path(
            "inverse_freeform_registered_brain.nii"
        )

        self.registered_atlas_img_path = self.make_reg_path(
            "registered_atlas.nii"
        )
        self.registered_hemispheres_img_path = self.make_reg_path(
            "registered_hemispheres.nii"
        )

        self.affine_matrix_path = self.make_reg_path("affine_matrix.txt")
        self.invert_affine_matrix_path = self.make_reg_path(
            "invert_affine_matrix.txt"
        )

        self.control_point_file_path = self.make_reg_path(
            "control_point_file.nii"
        )
        self.inverse_control_point_file_path = self.make_reg_path(
            "inverse_control_point_file.nii"
        )

        self.boundaries_file_path = self.make_reg_path("boundaries.nii")

        (
            self.tmp__affine_log_file_path,
            self.tmp__affine_error_path,
        ) = self.compute_reg_log_file_paths("affine")
        (
            self.tmp__freeform_log_file_path,
            self.tmp__freeform_error_file_path,
        ) = self.compute_reg_log_file_paths("freeform")
        (
            self.tmp__inverse_freeform_log_file_path,
            self.tmp__inverse_freeform_error_file_path,
        ) = self.compute_reg_log_file_paths("inverse_freeform")
        (
            self.tmp__segmentation_log_file,
            self.tmp__segmentation_error_file,
        ) = self.compute_reg_log_file_paths("segment")
        (
            self.tmp__invert_affine_log_file,
            self.tmp__invert_affine_error_file,
        ) = self.compute_reg_log_file_paths("invert_affine")

    def make_reg_path(self, basename):
        """
        Compute the absolute path of the destination file to
        self.registration_output_folder.

        :param str basename:
        :return: The path
        :rtype: str
        """
        return os.path.join(self.registration_output_folder, basename)

    def compute_reg_log_file_paths(self, basename):
        """
        Compute the path of the log and err file for the step corresponding
        to basename

        :param str basename:
        :return: log_file_path, error_file_path
        """

        log_file_template = os.path.join(
            self.registration_output_folder, "{}.log"
        )
        error_file_template = os.path.join(
            self.registration_output_folder, "{}.err"
        )
        log_file_path = log_file_template.format(basename)
        error_file_path = error_file_template.format(basename)
        return log_file_path, error_file_path



"""
registration_params
===================

The module to handle all the registration options and program binaries
"""


from amap.config.atlas import Atlas
from imlib.general.config import get_config_obj
from imlib.source.niftyreg_binaries import get_niftyreg_binaries, get_binary


class RegistrationParams:
    """
    A class to store and access the variables required for the registration
    including the paths of the different binaries and atlases.
    Options are typically stored as a tuple of (option_string, option_value)
    """

    def __init__(
        self,
        config_path,
        affine_n_steps=6,
        affine_use_n_steps=5,
        freeform_n_steps=6,
        freeform_use_n_steps=4,
        bending_energy_weight=0.95,
        grid_spacing_x=-10,
        smoothing_sigma_reference=-1.0,
        smoothing_sigma_floating=-1.0,
        histogram_n_bins_floating=128,
        histogram_n_bins_reference=128,
    ):
        self.config = get_config_obj(config_path)
        self.transform_program_path = self.__get_binary("transform")
        self.affine_reg_program_path = self.__get_binary("affine")
        self.freeform_reg_program_path = self.__get_binary("freeform")
        self.segmentation_program_path = self.__get_binary("segmentation")

        # affine (reg_aladin)
        self.affine_reg_pyramid_steps = ("-ln", affine_n_steps)
        self.affine_reg_used_pyramid_steps = ("-lp", affine_use_n_steps)

        # freeform (ref_f3d)
        self.freeform_reg_pyramid_steps = ("-ln", freeform_n_steps)
        self.freeform_reg_used_pyramid_steps = ("-lp", freeform_use_n_steps)

        self.freeform_reg_grid_spacing_x = ("-sx", grid_spacing_x)

        self.bending_energy_penalty_weight = ("-be", bending_energy_weight)

        self.reference_image_smoothing_sigma = (
            "-smooR",
            smoothing_sigma_reference,
        )
        self.floating_image_smoothing_sigma = (
            "-smooF",
            smoothing_sigma_floating,
        )

        self.reference_image_histo_n_bins = (
            "--rbn",
            histogram_n_bins_reference,
        )
        self.floating_image_histo_n_bins = ("--fbn", histogram_n_bins_floating)

        # segmentation (reg_resample)
        self.segmentation_interpolation_order = ("-inter", 0)

        # The atlas has been saved to the output folder
        atlas = Atlas(config_path)
        # self.atlas_path = atlas.get_path()
        # self.atlas_brain_path = atlas.get_brain_path()
        # self.hemispheres_path = atlas.get_hemispheres_path()

        pixel_sizes = atlas.get_pixel_sizes_from_config()
        self.atlas_x_pix_size = pixel_sizes["x"]
        self.atlas_y_pix_size = pixel_sizes["y"]
        self.atlas_z_pix_size = pixel_sizes["z"]

    def get_affine_reg_params(self):
        """
        Get the parameters (options) required for the affine registration step

        :return: The affine registration options.
        :rtype: list
        """
        affine_params = [
            self.affine_reg_pyramid_steps,
            self.affine_reg_used_pyramid_steps,
        ]
        return affine_params

    def get_freeform_reg_params(self):
        """
        Get the parameters (options) required for the freeform (elastic)
        registration step

        :return: The freeform registration options.
        :rtype: list
        """
        freeform_params = [
            self.freeform_reg_pyramid_steps,
            self.freeform_reg_used_pyramid_steps,
            self.freeform_reg_grid_spacing_x,
            self.bending_energy_penalty_weight,
            self.reference_image_smoothing_sigma,
            self.floating_image_smoothing_sigma,
            self.reference_image_histo_n_bins,
            self.floating_image_histo_n_bins,
        ]
        return freeform_params

    def get_segmentation_params(self):
        """
        Get the parameters (options) required for the segmentation step
        (propagation of transformation)

        :return: The affine registration options.
        :rtype: list
        """
        return [self.segmentation_interpolation_order]

    def format_param_pairs(self, params_pairs):
        """
        Format the list of params pairs into a string

        :param list params_pairs: A list of tuples of the form
            (option_string, option_value) (e.g. (-sx, 10))
        :return: The options as a formatted string
        :rtype: str
        """
        out = ""
        for param in params_pairs:
            out += "{} {} ".format(*param)
        return out

    def format_affine_params(self):
        """
        Generate the string of formatted affine registration options

        :return: The formatted string
        :rtype: str
        """
        return self.format_param_pairs(self.get_affine_reg_params())

    def format_freeform_params(self):
        """
        Generate the string of formatted freeform registration options

        :return: The formatted string
        :rtype: str
        """
        return self.format_param_pairs(self.get_freeform_reg_params())

    def format_segmentation_params(self):
        """
        Generate the string of formatted segmentation options

        :return: The formatted string
        :rtype: str
        """
        return self.format_param_pairs(self.get_segmentation_params())

    def __get_binary(self, program_type):
        """
        Get the path to the registration (from nifty_reg) program
        based on the type

        :param str program_type:
        :return: The program path
        :rtype: str
        """

        program_names = {
            "affine": "reg_aladin",
            "freeform": "reg_f3d",
            "segmentation": "reg_resample",
            "transform": "reg_transform",
        }
        program_name = program_names[program_type]
        nifty_reg_binaries_folder = get_niftyreg_binaries()

        program_path = get_binary(nifty_reg_binaries_folder, program_name)

        return program_path



from setuptools import setup, find_namespace_packages
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

requirements = [
    "nibabel",
    "numpy",
    "configparser",
    "pandas",
    "scikit-image",
    "tqdm",
    "natsort",
    "multiprocessing-logging",
    "configobj",
    "slurmio",
    "brainio>=0.0.10",
    "fancylog",
    "micrometa",
    "imlib",
    "napari>=0.2.8",
    "scikit-image",
    "luddite",
]


setup(
    name="amap",
    version="0.1.14rc0",
    description="Automated mouse atlas propagation",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=requirements,
    extras_require={
        "dev": [
            "sphinx",
            "recommonmark",
            "sphinx_rtd_theme",
            "pydoc-markdown",
            "black",
            "pytest-cov",
            "pytest",
            "gitpython",
            "coveralls",
            "coverage<=4.5.4",
        ]
    },
    python_requires=">=3.6",
    packages=find_namespace_packages(exclude=("docs", "tests*")),
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "amap = amap.cli:main",
            "amap_gui = amap.cli:gui",
            "amap_download = amap.download.cli:main",
            "amap_vis = amap.vis.vis:main",
        ]
    },
    project_urls={
        "Source Code": "https://github.com/SainsburyWellcomeCentre/amap-python",
        "Bug Tracker": "https://github.com/SainsburyWellcomeCentre/amap-python/issues",
    },
    author="Adam Tyson, Charly Rousseau, Christian Niedworok",
    author_email="adam.tyson@ucl.ac.uk",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Operating System :: POSIX :: Linux",
        "Operating System :: Microsoft :: Windows :: Windows 10",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
    ],
    zip_safe=False,
)



import os

from imlib.general.system import ensure_directory_exists
from amap.download.download import download

default_atlas_name = "brain.nii"

# allen_2017 (default) and allen_2017_10um are the same
atlas_urls = {
    "allen_2017_10um": "https://gin.g-node.org/cellfinder/atlas/raw/master/allen_2017_10um.tar.gz",
    "allen_2017_100um": "https://gin.g-node.org/cellfinder/atlas/raw/master/allen_2017_100um.tar.gz",
}

download_requirements_gb = {
    "allen_2017_10um": 1.8,
    "allen_2017_100um": 0.003,
}

extract_requirements_gb = {
    "allen_2017_10um": 11.5,
    "allen_2017_100um": 0.015,
}


def main(atlas, atlas_dir, download_path):
    if not os.path.exists(os.path.join(atlas_dir, atlas, default_atlas_name)):
        ensure_directory_exists(atlas_dir)

        download(
            download_path,
            atlas_urls[atlas],
            atlas,
            install_path=atlas_dir,
            download_requires=download_requirements_gb[atlas],
            extract_requires=extract_requirements_gb[atlas],
        )

    else:
        print(
            f"Atlas already exists at " f"{atlas_dir}." f" Skipping download"
        )
